﻿(***************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                           BIALYSTOK 2017                               **
**                                                                        **
**                               CONFIG                                   **
**                      Wersja z dnia : 2018.06.25                        **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
***************************************************************************)

(*
   Specjalne podziękowania dla Nataszy Biecek, jej pomoc miała
   znaczenie w powstanoiu tego programu.
*)

unit configunit ;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, ueled, SdpoSerial, Forms, Controls,
  Graphics, Dialogs, StdCtrls, ExtCtrls, synaser;

const
  CyclicBufferSize             = 256 ;

type
  CyclicBufferRecT             = record
                                   FrIndex        : word ;
                                   ToIndex        : word ;
                                   Buffer         : array [ 0 .. CyclicBufferSize - 1 ] of char ;
                                 end (* record *) ;

  { TConfigurationUnit }

  TConfigurationUnit = class(TForm)
    HelloButton                : TButton ;
    GetVerButton               : TButton ;
    ReinitButton               : TButton ;
    SaveEEButton               : TButton ;
    ResetEEButton              : TButton ;
    GetIDButton                : TButton ;
    ClearButton                : TButton ;
    SendReplyButton            : TButton ;
    ClosePortButton            : TButton ;
    OpenPortButton             : TButton ;
    SendButton                 : TButton ;
    ExitButton                 : TButton ;
    DestDeviceIDLabel          : TLabel ;
    CommandLabel               : TLabel ;
    SendToSerialLabel          : TLabel ;
    RecvFrtomSerialLabel       : TLabel ;
    SerialPortComboBoxLabel    : TLabel ;
    CommandEdit                : TEdit ;
    DestDeviceIDEdit           : TEdit ;
    RecvFromSerialMemo         : TMemo ;
    SerialChannel              : TSdpoSerial ;
    SerialPortComboBox         : TComboBox ;
    SendToSerialMemo           : TMemo ;
    GeneralTimer               : TTimer ;
    ConnectLED                 : TuELED ;
    procedure ClearButtonClick ( Sender : TObject ) ;
    procedure ClosePortButtonClick ( Sender : TObject ) ;
    procedure ExitButtonClick ( Sender : TObject ) ;
    procedure FormCreate ( Sender : TObject ) ;
    procedure GeneralTimerTimer ( Sender : TObject ) ;
    procedure GetIDButtonClick ( Sender: TObject ) ;
    procedure GetVerButtonClick ( Sender: TObject ) ;
    procedure HelloButtonClick( Sender: TObject ) ;
    procedure OpenPortButtonClick ( Sender : TObject ) ;
    procedure ReinitButtonClick(Sender: TObject);
    procedure ResetEEButtonClick ( Sender: TObject ) ;
    procedure SaveEEButtonClick ( Sender: TObject ) ;
    procedure SendButtonClick ( Sender : TObject ) ;
    procedure SendReplyButtonClick ( Sender : TObject ) ;
    procedure EnableTotalButtons ;
    procedure DisableTotalButtons ;
    procedure ExecuteCommand ( Command : string ) ;
    procedure AddToCyclic ( Ch : char ) ;
    function GetCh ( ) : char ;
  private
    { private declarations }
  public
    { public declarations }
    CyclicBuffer                : CyclicBufferRecT ;
    RecvCommand                 : string ;
  end;

var
  ConfigurationUnit : TConfigurationUnit ;

implementation

{$R *.lfm}

{ TConfigurationUnit }

procedure TConfigurationUnit.EnableTotalButtons ;
begin (* TConfigurationUnit.EnableTotalButtons *)
  OpenPortButton . Enabled := false ;
  ClosePortButton . Enabled := true ;
  ExitButton . Enabled := false ;
  SendButton . Enabled := true ;
  SendReplyButton . Enabled := true ;
  HelloButton . Enabled := true ;
  GetVerButton . Enabled := true ;
  ReinitButton . Enabled := true ;
  SaveEEButton . Enabled := true ;
  ResetEEButton . Enabled := true ;
  GetIDButton . Enabled := true ;
end (* TConfigurationUnit.EnableTotalButtons *) ;


procedure TConfigurationUnit.DisableTotalButtons ;
begin (* TConfigurationUnit.DisableTotalButtons *)
  OpenPortButton . Enabled := true ;
  ClosePortButton . Enabled := false ;
  ExitButton . Enabled := true ;
  SendButton . Enabled := false ;
  SendReplyButton . Enabled := false ;
  HelloButton . Enabled := false ;
  GetVerButton . Enabled := false ;
  ReinitButton . Enabled := false ;
  SaveEEButton . Enabled := false ;
  ResetEEButton . Enabled := false ;
  GetIDButton . Enabled := false ;
end (* TConfigurationUnit.DisableTotalButtons *) ;


procedure TConfigurationUnit.FormCreate ( Sender : TObject ) ;
begin (* TConfigurationUnit.FormCreate *)
  SerialPortComboBox . Items . CommaText := GetSerialPortNames ;
  SerialPortComboBox . Text := SerialPortComboBox . Items [ 0 ] ;
  GeneralTimer . Enabled := false ;
  DisableTotalButtons ;
end (* TConfigurationUnit.FormCreate *) ;


procedure TConfigurationUnit.OpenPortButtonClick ( Sender : TObject ) ;
var
  OpenFail             : boolean ;
begin (* TConfigurationUnit.OpenPortButtonClick *)
  OpenFail := false ;
  SerialChannel . Device := SerialPortComboBox . Text ;
  try
    SerialChannel . Open ;
  except
    ShowMessage ( string ('Port komunikacyjny ' ) + SerialPortComboBox . Text +
                  string ( ' nie jest dostępny.'  ) ) ;
    OpenFail := true ;
  end (* try *) ;
  if OpenFail then
  begin (* 1 *)
    SerialChannel . Close ;
    exit ;
  end (* 1 *) ;
  RecvFromSerialMemo . Clear ;
  SendToSerialMemo . Clear ;
  OpenPortButton . Enabled := false ;
  GeneralTimer. Enabled := true ;
  ConnectLED . Color := clGreen ;
  ConnectLED . Active := true ;
  GeneralTimer. Enabled := true ;
  EnableTotalButtons ;
  CyclicBuffer . FrIndex := 0 ;
  CyclicBuffer . ToIndex := 0 ;
  RecvCommand := string ( '' ) ;
end (* TConfigurationUnit.OpenPortButtonClick *) ;


procedure TConfigurationUnit.ClosePortButtonClick ( Sender : TObject ) ;
begin (* TConfigurationUnit.ClosePortButtonClick *)
  ConnectLED . Active := false ;
  GeneralTimer . Enabled := false ;
  SerialChannel . Close ;
  DisableTotalButtons ;
end (* TConfigurationUnit.ClosePortButtonClick *) ;

procedure TConfigurationUnit.ClearButtonClick ( Sender : TObject ) ;
begin (* TConfigurationUnit.ClearButtonClick *)
  SendToSerialMemo . Lines . Clear ;
  RecvFromSerialMemo . Lines . Clear ;
end (* TConfigurationUnit.ClearButtonClick *) ;


procedure TConfigurationUnit.ExecuteCommand ( Command : string ) ;
var
  CommandToSend          : string ;
  DestinationID          : string ;
begin (* TConfigurationUnit.ExecuteCommand *)
  DestinationID := DestDeviceIDEdit . Text ;
  DestinationID := DestinationID . Trim ;
  if DestinationID . IsEmpty then
  begin (* 2 *)
    DestDeviceIDEdit . Text := string ( 'GENERAL' ) ;
    DestinationID := DestDeviceIDEdit . Text ;
  end (* 2 *) ;
  CommandToSend := DestinationID + string ( ':COMP:' ) + Command ;
  SendToSerialMemo . Lines . Add ( CommandToSend ) ;
  CommandToSend := CommandToSend + string ( chr ( $0D ) ) ;
  SerialChannel . WriteData ( CommandToSend ) ;
end (* TConfigurationUnit.ExecuteCommand *) ;


procedure TConfigurationUnit.SendButtonClick ( Sender : TObject ) ;
var
  Command                : string ;
begin (* TConfigurationUnit.SendButtonClick *)
  Command := CommandEdit . Text ;
  Command := Command . Trim ;
  if not Command . IsEmpty then
  begin (* 1 *)
    ExecuteCommand ( Command ) ;
  end (* 1 *) ;
end (* TConfigurationUnit.SendButtonClick *) ;


procedure TConfigurationUnit.SendReplyButtonClick ( Sender : TObject ) ;
var
  Command                : string ;
begin (* TConfigurationUnit.SendReplyButtonClick *)
  Command := string ( 'REPLY' ) ;
  ExecuteCommand ( Command ) ;
end (* TConfigurationUnit.SendReplyButtonClick *) ;


procedure TConfigurationUnit.GetIDButtonClick ( Sender : TObject ) ;
var
  Command                : string ;
begin (* TConfigurationUnit.GetIDButtonClick *)
  Command := string ( 'GETID' ) ;
  ExecuteCommand ( Command ) ;
end (* TConfigurationUnit.GetIDButtonClick *) ;


procedure TConfigurationUnit.GetVerButtonClick ( Sender : TObject ) ;
var
  Command                : string ;
begin (* TConfigurationUnit.GetVerButtonClick *)
  Command := string ( 'GETVER' ) ;
  ExecuteCommand ( Command ) ;
end (* TConfigurationUnit.GetVerButtonClick *) ;


procedure TConfigurationUnit.HelloButtonClick ( Sender : TObject ) ;
var
  Command                : string ;
begin (* TConfigurationUnit.HelloButtonClick *)
  Command := string ( 'HELLO' ) ;
  ExecuteCommand ( Command ) ;
end (* TConfigurationUnit.HelloButtonClick *) ;


procedure TConfigurationUnit.ReinitButtonClick ( Sender : TObject ) ;
var
  Command                : string ;
begin (* TConfigurationUnit.ReinitButtonClick *)
  Command := string ( 'REINIT' ) ;
  ExecuteCommand ( Command ) ;
end (* TConfigurationUnit.ReinitButtonClick *) ;


procedure TConfigurationUnit.ResetEEButtonClick ( Sender : TObject ) ;
var
  Command                : string ;
begin (* TConfigurationUnit.ResetEEButtonClick *)
  Command := string ( 'RESEE' ) ;
  ExecuteCommand ( Command ) ;
end (* TConfigurationUnit.ResetEEButtonClick *) ;


procedure TConfigurationUnit.SaveEEButtonClick ( Sender : TObject ) ;
var
  Command                : string ;
begin (* TConfigurationUnit.SaveEEButtonClick *)
  Command := string ( 'SAVEEE' ) ;
  ExecuteCommand ( Command ) ;
end (* TConfigurationUnit.SaveEEButtonClick *) ;


procedure TConfigurationUnit.ExitButtonClick ( Sender : TObject ) ;
begin (* TConfigurationUnit.ExitButtonClick *)
  Application . Terminate ;
end (* TConfigurationUnit.ExitButtonClick *) ;


procedure TConfigurationUnit.AddToCyclic ( Ch : char ) ;
begin (* TConfigurationUnit.AddToCyclic *)
  CyclicBuffer . Buffer [ CyclicBuffer . ToIndex ] := Ch ;
  CyclicBuffer . ToIndex := CyclicBuffer . ToIndex + 1 ;
  if CyclicBuffer . ToIndex >= CyclicBufferSize then
    CyclicBuffer . ToIndex := 0 ;
end (* TConfigurationUnit.AddToCyclic *) ;


function TConfigurationUnit.GetCh ( ) : char ;
var
  Ch                      : char ;
begin (* TConfigurationUnit.GetCh *)
  if CyclicBuffer . FrIndex = CyclicBuffer . ToIndex then
  begin (* 1 *)
    Ch := chr ( 0 ) ;
  end (* 1 *)
  else
  begin (* 1 *)
    Ch := CyclicBuffer . Buffer [ CyclicBuffer . FrIndex ] ;
    CyclicBuffer . FrIndex := CyclicBuffer . FrIndex + 1 ;
    if CyclicBuffer . FrIndex >= CyclicBufferSize then
      CyclicBuffer . FrIndex := 0 ;
  end (* 1 *) ;
  Result := Ch ;
end (* TConfigurationUnit.GetCh *) ;


procedure TConfigurationUnit.GeneralTimerTimer ( Sender : TObject ) ;
var
  Command                : string ;
  Loop                   : word ;
  StringSize             : word ;
  Ch                     : char ;
begin (* TConfigurationUnit.GeneralTimerTimer *)
  GeneralTimer . Enabled := false ;
  if SerialChannel . DataAvailable then
  begin (* 1 *)
    Command := SerialChannel . ReadData ;
    StringSize := Command . Length ;
    for Loop := 1 to StringSize do
    begin (* 2 *)
      Ch := Command [ Loop ] ;
      if Ch < ' ' then
      begin (* 3 *)
        if Ch = chr ( $0D ) then
          AddToCyclic ( Ch ) ;
      end (* 3 *)
      else
      begin (* 3 *)
        AddToCyclic ( Ch ) ;
      end (* 3 *) ;
    end (* 2 *) ;
  end (* 1 *) ;
  while CyclicBuffer . FrIndex <> CyclicBuffer . ToIndex do
  begin (* 1 *)
    Ch := GetCh ( ) ;
    if Ch = chr ( $0D ) then
    begin (* 2 *)
      RecvFromSerialMemo . Lines . Add ( RecvCommand ) ;
      RecvCommand := string ( '' ) ;
    end (* 2 *)
    else
    begin (* 2 *)
      RecvCommand := RecvCommand + Ch ;
    end (* 2 *) ;
  end (* 1 *) ;
  GeneralTimer . Enabled := true ;
end (* TConfigurationUnit.GeneralTimerTimer *) ;


end.

